unit RCom;
{ Connecting to R via COM using Neuwirth's StatConnectorSrvLib 
  See http://cran.r-project.org/contrib/extra/dcom
  By dieter.menne@menne-biomed.de
}

interface
uses
  Windows, Messages, SysUtils,Classes, Dialogs,  variants, STATCONNECTORSRVLib_TLB,Forms;

  type
    TDoubleMatrix= array of array of double;
    TRConnector= class(TStatConnector)
    private
      FSinked: boolean;
      FWorkingDirectory:        string;
      function GetRSymbol(index: string): OleVariant;
      procedure SetRSymbol(index: string; const Value: OleVariant);
      procedure SetWorkingDirectory(const Value: string);
    public
      constructor Create(AOwner: TComponent);override;
      destructor Destroy; override;
      procedure Clear;
      property Sym[index: string]: OleVariant read GetRSymbol write SetRSymbol; default;
      procedure Source(FileName: string);
      property WorkingDirectory: string read FWorkingDirectory write SetWorkingDirectory;
      procedure GetDoubleVector(Symbol: string; var Len: integer;var Vec: array of double);
      procedure GetStringVector(Symbol: string; var Len:integer;
                                     var Vec: array of string);
      procedure GetIntVector(Symbol: string; var Len: integer;
        var Vec: array of integer);
      procedure SetDoubleVector(Symbol: string; Len: integer;
        const Vec: array of double);
      procedure SetStringVector(Symbol: string;Len: integer;
                                   const Vec: array of string);
      procedure SetIntVector(Symbol: string; Len: integer;
        const Vec: array of integer);
      procedure SetDoubleMatrix(Symbol: string;rows,cols: integer;
                             const Mat: TDoubleMatrix);
      procedure GetDoubleMatrix(Symbol: string; var rows, cols: integer;
        var Mat: TDoubleMatrix);
      procedure Sink(FileName: string='');
      procedure Print(Symbol: string);
      procedure CatLn(Symbol: string);
      procedure EvaluateNoReturn(const bstrExpression: string);
      procedure ENR(const bstrExpression: string); // Short form of EvaluatNop.
      function Evaluate(const bstrExpression: string): OleVariant;
  end;

implementation


constructor TRConnector.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Init('R');
  //SetWorkingDirectory(ExtractFilePath(Application.ExeName));
end;

destructor TRConnector.Destroy;
begin
  ENR('graphics.off()');
  Close;
  inherited Destroy;
end;

function TRConnector.GetRSymbol(index: string): OleVariant;
begin
  try
    Result:=GetSymbol(index);
  except
    raise Exception.CreateFmt('Cannot get R-symbol <%s>',[index])
  end;
end;

procedure TRConnector.SetRSymbol(index: string; const Value: OleVariant);
begin
  try
    SetSymbol(index,Value);
  except
    raise Exception.CreateFmt('Cannot set R-symbol <%s>',[index])
  end;
end;

procedure TRConnector.SetWorkingDirectory(const Value: string);
var i: integer;
begin
  FWorkingDirectory := Value;
  for i:=1 to Length(FWorkingDirectory) do
    if FWorkingDirectory[i]='\' then
      FWorkingDirectory[i]:='/';
  if FWorkingDirectory[Length(FWorkingDirectory)] <> '/' then
    FWorkingDirectory:=FWorkingDirectory+'/';
  EvaluateNoReturn(Format('setwd("%s")',[FWorkingDirectory]));
end;

procedure TRConnector.Source(FileName: string);
begin
  // Source a filename in the working directory
  FileName:=FWorkingDirectory+FileName;
  if Not FileExists(FileName) then
    raise Exception.CreateFmt('File %s not found',[FileName]);
  //writeln(Format('source("%s")',[FileName]));
  try
    EvaluateNoReturn(Format('source("%s")',[FileName]));
  except
    raise Exception.CreateFmt('Error sourcing %s to R',[FileName]);
  end;
end;

procedure TRConnector.GetDoubleVector(Symbol: string; var Len:integer;
                                     var Vec: array of double);
var v: OleVariant;
    i,Dim,VType: integer;
begin
  v:=GetRSymbol(Symbol);
  VType:=VarType(v);
  if ((VType and varArray) = 0) or
     ((VType and (varInteger or varSingle or varDouble)) = 0) then
    raise Exception.CreateFmt('%s is not of array type int,single or double',
          [Symbol]);
  Dim:=VarArrayDimCount(v);
  if Dim <> 1 then
    raise Exception.CreateFmt('%s is no 1-dimensional vector',[Symbol]);
  Len:=VarArrayHighBound(v,1)+1;
  if Length(Vec) < len then
    raise Exception.CreateFmt('Not enough allocated space for vector %s',[Symbol]);
  for i:=0 to Len-1 do
    Vec[i]:=v[i];
end;

procedure TRConnector.GetStringVector(Symbol: string; var Len:integer;
                                     var Vec: array of string);
var v: OleVariant;
    i,Dim,VType: integer;
begin
  v:=GetRSymbol(Symbol);
  VType:=VarType(v);
  if ((VType and varArray) = 0) or
     ((VType and varOleStr) = 0) then
    raise Exception.CreateFmt('%s is not of array type string',
          [Symbol]);
  Dim:=VarArrayDimCount(v);
  if Dim <> 1 then
    raise Exception.CreateFmt('%s is no 1-dimensional vector',[Symbol]);
  Len:=VarArrayHighBound(v,1)+1;
  if Length(Vec) < len then
    raise Exception.CreateFmt('Not enough allocated space for vector %s',[Symbol]);
  for i:=0 to Len-1 do
    Vec[i]:=v[i];
end;

procedure TRConnector.GetIntVector(Symbol: string; var Len:integer;
                                     var Vec: array of integer);
var v: OleVariant;
    i,Dim,VType: integer;
begin
  v:=GetRSymbol(Symbol);
  VType:=VarType(v);
  if ((VType and varArray) = 0) or
     ((VType and varInteger) = 0) then
    raise Exception.CreateFmt('%s is not of array type integer',
          [Symbol]);
  Dim:=VarArrayDimCount(v);
  if Dim <> 1 then
    raise Exception.CreateFmt('%s is no 1-dimensional vector',[Symbol]);
  Len:=VarArrayHighBound(v,1)+1;
  if Length(Vec) < len then
    raise Exception.CreateFmt('Not enough allocated space for vector %s',[Symbol]);
  for i:=0 to Len-1 do
    Vec[i]:=v[i];
end;

procedure TRConnector.SetDoubleVector(Symbol: string;Len: integer;
                                      const Vec: array of double);
var v: OleVariant;
    i: integer;
begin
  v:=VarArrayCreate([0,Len-1], varDouble);
  for i:=0 to Len-1 do
    v[i]:=Vec[i];
  try
    SetRSymbol(Symbol,v);
  except
    raise Exception.CreateFmt('Erros SetDoubleVector %s',[Symbol]);
  end;
end;

procedure TRConnector.SetIntVector(Symbol: string;Len: integer;
                                   const Vec: array of integer);
var v: OleVariant;
    i: integer;
begin
  v:=VarArrayCreate([0,Len-1], varInteger);
  for i:=0 to Len-1 do
    v[i]:=Vec[i];
  try
    SetRSymbol(Symbol,v);
  except
    raise Exception.CreateFmt('Erros SetIntVector %s',[Symbol]);
  end;
end;

procedure TRConnector.SetStringVector(Symbol: string;Len: integer;
                                   const Vec: array of string);
var v: OleVariant;
    i: integer;
begin
  v:=VarArrayCreate([0,Len-1], varOleStr);
  for i:=0 to Len-1 do
    v[i]:=Vec[i];
  try
    SetRSymbol(Symbol,v);
  except
    raise Exception.CreateFmt('Erros SetIntVector %s',[Symbol]);
  end;
end;

procedure TRConnector.SetDoubleMatrix(Symbol: string;rows,cols: integer;
                                  const Mat: TDoubleMatrix);
var v: OleVariant;
    j,i: integer;
begin
  v:=VarArrayCreate([0,rows-1,0,cols-1], varDouble);
  for j:=0 to rows-1 do
    for i:=0 to cols-1 do
      v[j,i]:=mat[j,i];
  try
    SetRSymbol(Symbol,v);
  except
    raise Exception.CreateFmt('Erros SetDoubleMatrix %s',[Symbol]);
  end;
end;


procedure TRConnector.GetDoubleMatrix(Symbol: string; var rows,cols: integer;
                                     var Mat: TDoubleMatrix);
var v: OleVariant;
    j,i,Dim,VType: integer;
begin
  v:=GetRSymbol(Symbol);
  VType:=VarType(v);
  if ((VType and varArray) = 0) or
     ((VType and (varInteger or varSingle or varDouble)) = 0) then
    raise Exception.CreateFmt('%s is not of array type int,single or double',
          [Symbol]);
  Dim:=VarArrayDimCount(v);
  if Dim <>2  then
    raise Exception.CreateFmt('%s is no matrix',[Symbol]);
  rows:=VarArrayHighBound(v,1)+1;
  cols:=VarArrayHighBound(v,2)+1;
  if (Length(Mat) < rows) or (Length(Mat[0]) < cols) then
    raise Exception.CreateFmt('Not enough allocated space for matrix %s',[Symbol]);
  for j:=0 to rows-1 do
    for i:=0 to cols-1 do
      mat[j,i]:=v[j,i];
end;


procedure TRConnector.Clear;
begin
  EvaluateNoReturn('rm(list=ls(all=T))');
end;

procedure TRConnector.Sink(FileName: string);
begin
  // We avoid stacked syncs
  if (FileName= '') or FSinked then
    EvaluateNoReturn('sink();'); // remove old sink
  if FileName <> '' then
  begin
    EvaluateNoReturn(Format('sink("%s");',[FileName]));
    FSinked:=True;
  end
  else
    FSinked:=False;
end;


procedure TRConnector.CatLn(Symbol: string);
begin
  EvaluateNoReturn(Format('cat(%s,"\n",sep="\t")', [Symbol]));
end;

procedure TRConnector.Print(Symbol: string);
begin
  EvaluateNoReturn(Format('print(%s)', [Symbol]));
end;

procedure TRConnector.EvaluateNoReturn(const bstrExpression: string);
begin
  try
    inherited EvaluateNoReturn(bstrExpression);
  except
    raise Exception.Create('Error EvaluateNoReturn "'+bstrExpression+'"');
  end;
end;

// Same as EvaluateNoReturn, but shorter
procedure TRConnector.ENR(const bstrExpression: string);
begin
  try
    inherited EvaluateNoReturn(bstrExpression);
  except
    raise Exception.Create('Error EvaluateNoReturn "'+bstrExpression+'"');
  end;
end;

function TRConnector.Evaluate(const bstrExpression: string): OleVariant;
begin
  try
    Result:=inherited Evaluate(bstrExpression);
  except
    raise Exception.Create('Error Evaluate <'+bstrExpression+'>');
  end;
end;

end.

